/*******************************************************************************/
/**	Description: 															  **/
/**		glcd.c - Graphical LCD driver functions								  **/
/*******************************************************************************/
/**  Compiler	: 	Keil CARM V2.4											  **/
/**	 Target 	:	Philips LPC2000											  **/
/**	 Code portions Copyright (C) 2004 Martin THOMAS							  **/
/**  Author		: 	Marko Pavlin, 											  **/
/**					marko.pavlin@guest.arnes.si								  **/
/*******************************************************************************/
/**	Revision history: 														  **/
/**	-----------------														  **/
/**	  v0.0 21.12.2005 - file created										  **/
/**	  v1.0 31.1.2006 - Version 1.0 tested and released to project			  **/
/**	  v1.1 3.2.2006 - Code cleanup and comments added						  **/
/**																			  **/
/*******************************************************************************/
			    
#include "glcd.h"
#include "fonts.h"
#include "glcdhw.h"

/* local defines */
#define GLCD_BUSOUT 1
#define GLCD_MODECMD  0
#define GLCD_MODEDATA 1
#define GLCD_SET_PAGE       0xB0
#define GLCD_SET_ADDRESS	0x10
#define GLCD_X_BYTES (GLCD_MAXADDRESS)
#define GLCD_Y_BYTES (GLCD_MAXPAGE)


static unsigned char glcd_Inverted;
static unsigned char glcd_Buffer[GLCD_Y_BYTES][GLCD_X_BYTES];
static const unsigned char glcd_MaskArray[8] =
         {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};


/* prototypes */
//static void glcd_StrobeEnable(void);
static void glcd_SetPage(unsigned char page);
static void glcd_SetAddress(unsigned char address);
static void glcd_WriteData(unsigned char byte);

static void glcd_FillBuffer(unsigned char pattern);

void glcd_Glyph(unsigned char left, unsigned char top,
			   unsigned char width, unsigned char height,
			   const unsigned char *glyph, unsigned char store_width);




/*********************************************************************
 * Function:        glcd_WriteDataCmd(unsigned char byte, unsigned char mode)
 * PreCondition:    LCD initialized
 * Input:           data and mode (GLCD_MODECMD, GLCD_MODEDATA) 
 * Output:          -
 * Overview:        Send byte of data or command to LCD module
 * Note:            None
 ********************************************************************/
void glcd_WriteDataCmd(unsigned char byte, unsigned char mode)
{
  switch (mode) {
    case GLCD_MODECMD:
      hal_LCD_init();  	// setup SPI for LCD
      hal_LCD_CS(0);	 // Salve select for LCD
      hal_LCD_RS(0); // command
   	  hal_LCD_transfer(byte);	// send byte
      hal_LCD_CS(1);	 // Salve select for LCD
	break;

	case GLCD_MODEDATA:
      hal_LCD_init();  	// setup SPI for LCD
      hal_LCD_CS(0);	 // Salve select for LCD
      hal_LCD_RS(1); // data
   	  hal_LCD_transfer(byte);	// send byte
      hal_LCD_CS(1);	 // Salve select for LCD

	break;
  }
}

/*********************************************************************
 * Function:        glcd_WriteCommand(unsigned char byte)
 * PreCondition:    LCD initialized
 * Input:           command byte 
 * Output:          -
 * Overview:        Send command to LCD module
 * Note:            None
 ********************************************************************/
void glcd_WriteCommand(unsigned char byte)
{
	glcd_WriteDataCmd(byte, GLCD_MODECMD);
}

/*********************************************************************
 * Function:        glcd_WriteData(unsigned char byte)
 * PreCondition:    LCD initialized
 * Input:           data byte 
 * Output:          -
 * Overview:        Send data to LCD module
 * Note:            None
 ********************************************************************/
void glcd_WriteData(unsigned char byte)
{
	glcd_WriteDataCmd(byte, GLCD_MODEDATA);
}

/*********************************************************************
 * Function:        glcd_SetPage(unsigned char page)
 * PreCondition:    LCD initialized
 * Input:           page number 
 * Output:          -
 * Overview:        Set page number in LCD module
 * Note:            Page is horizontal strip, 8 pixels high and
 *					GLCD_MAXADDRESS pixels wide
 ********************************************************************/
void glcd_SetPage(unsigned char page)
{
	glcd_WriteCommand(GLCD_SET_PAGE+page);
}

/*********************************************************************
 * Function:        glcd_SetAddress(unsigned char address)
 * PreCondition:    LCD initialized
 * Input:           address number 
 * Output:          -
 * Overview:        Set address in LCD module
 * Note:            Address is horizontal position within page
 ********************************************************************/
void glcd_SetAddress(unsigned char address)
{
	glcd_WriteCommand(GLCD_SET_ADDRESS+(address&0xf0)>>4);
	glcd_WriteCommand(GLCD_SET_ADDRESS+(address&0x0f));
}

/*********************************************************************
 * Function:        glcd_Init(void)
 * PreCondition:    SPI and RTOS initialized
 * Input:           - 
 * Output:          -
 * Overview:        Initialize LCD module
 * Note:            Reset pause is implemented with os_dly_wait()
 ********************************************************************/
void glcd_Init(void)
{	
  unsigned int i,j;

  hal_LCD_init();  	// setup SPI for LCD
  hal_LCD_CS(0);	 // Salve select for LCD

/*
  IOCLR0 = LCD_RESET_BIT;
  os_dly_wait (5);
  IOSET0 = LCD_RESET_BIT;
  os_dly_wait (5);
*/

  /* Init LCD */

  hal_LCD_RS(0); // command

  hal_LCD_transfer(0xA2);// LCD bias Set (1/9 bias)
  hal_LCD_transfer(0xA0);// ADC select 
  hal_LCD_transfer(0xC8);// Common output mode 
  hal_LCD_transfer(0x27);//Internal resistor ratio   (max)

  hal_LCD_transfer(0x81);// Electronic volume control
  hal_LCD_transfer(0x1c);// V0 voltage ratio (large)

  hal_LCD_transfer(0xF8); //booster ratio select mode set
  hal_LCD_transfer(0x03); // 6x


  hal_LCD_transfer(0x2f);//power control set 
  hal_LCD_transfer(0xAF);//display on
  //hal_LCD_transfer(0xA7);//display REVERSE
  hal_LCD_transfer(0x40);//Start line = 0

  for (j=0; j<8; j++) {
    glcd_WriteCommand(0xb0+j);
	glcd_WriteCommand(0x10);
	glcd_WriteCommand(0x00);
    for (i=0; i<128; i++) 
      glcd_WriteData(i);
  }

  hal_LCD_CS(1);	 // Salve select for LCD


}

/*********************************************************************
 * Function:        glcd_SetInvert(unsigned char mode)
 * PreCondition:    LCD initialized
 * Input:           mode (GLCD_MODE_INVERT or GLCD_MODE_NORMAL) 
 * Output:          -
 * Overview:        Set writing mode 
 * Note:            -
 ********************************************************************/
void glcd_SetInvert(unsigned char mode)
{
	glcd_Inverted = mode;
}

/*********************************************************************
 * Function:        glcd_FillBuffer(unsigned char pattern)
 * PreCondition:    LCD initialized
 * Input:           fill pattern  
 * Output:          -
 * Overview:        Fill local LCD buffer with pattern 
 * Note:            -
 ********************************************************************/
void glcd_FillBuffer(unsigned char pattern)
{
	unsigned int x,y;
	
	for (y=0;y<GLCD_Y_BYTES;y++)
		for (x=0;x<GLCD_X_BYTES;x++) glcd_Buffer[y][x]=pattern;
}

/*********************************************************************
 * Function:        glcd_ClrScr(unsigned char bufferAndDisplay)
 * PreCondition:    LCD initialized
 * Input:           bufferAndDisplay (boolean), if 1 = clear LCD too  
 * Output:          -
 * Overview:        clear LCD buffer and display 
 * Note:            -
 ********************************************************************/
void glcd_ClrScr(unsigned char bufferAndDisplay)
{
	glcd_FillBuffer(0x00);
	if ( bufferAndDisplay ) glcd_UpdateAll();
}



/*********************************************************************
 * Function:        glcd_Update(unsigned int top, unsigned int bottom)
 * PreCondition:    LCD initialized
 * Input:           top     top line of area to update.         
 *					bottom  bottom line of area to update.
 * Output:          -
 * Overview:        Copy buffer-RAM to display-RAM 
 * Note:            -
 ********************************************************************/
void glcd_Update(unsigned int top, unsigned int bottom)
{
	unsigned char x, y;
	unsigned char yt, yb;
	unsigned char data;
	
	yt = top >> 3;				/* setup bytes of range */
	yb = bottom >> 3;

		for (y=yt;y<=yb;y++) {
			glcd_SetPage(y);
			glcd_SetAddress(0);
			for (x=0;x<GLCD_MAXADDRESS;x++) {
				data = glcd_Buffer[y][x];
				if (glcd_Inverted) data=~data;
				glcd_WriteData(data);
			}
		}
	
}

/*********************************************************************
 * Function:        glcd_Dot(unsigned char x, unsigned char y, unsigned char mode)
 * PreCondition:    LCD initialized
 * Input:           x,y - dot coordinate         
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Set a single dot within LCD buffer
 * Note:            -
 ********************************************************************/
void glcd_Dot(unsigned char x, unsigned char y, unsigned char mode) {
	unsigned char bitnum, bitmask, yByte;
	
	if ( (x>=GLCD_RIGHT) || (y>=GLCD_BOTTOM) ) return;
	
	yByte   = y >> 3; // / 8
	bitnum  = y & 0x07;
	bitmask = glcd_MaskArray[bitnum];
	switch (mode) {
		case GLCD_MODE_SET:
			glcd_Buffer[yByte][x] |= bitmask;
			break;
		case GLCD_MODE_CLEAR:
			glcd_Buffer[yByte][x] &= ~bitmask;
			break;
		case GLCD_MODE_XOR:
			glcd_Buffer[yByte][x] ^= bitmask;
			break;
		default: break;
	}
}

/*********************************************************************
 * Function:        void glcd_Line( unsigned char x1, unsigned char y1, 
						unsigned char x2, unsigned char y2,
						unsigned char mode )  
 * PreCondition:    LCD initialized
 * Input:           x1,y1 - line start coordinate
 *					x2,y2 - line end coordinate
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a line 
 * Note:            -
 ********************************************************************/
void glcd_Line( unsigned char x1, unsigned char y1, 
		unsigned char x2, unsigned char y2,
		unsigned char mode ) 
{
	unsigned char length, xTmp, yTmp, i, y, yAlt;
	int m;
	
	if(x1 == x2) {									// vertical line
		// x1|y1 must be the upper point
		if(y1 > y2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = y2-y1;
		for(i=0; i<=length; i++) {
			glcd_Dot(x1, y1+i, mode);
		}
	} else if(y1 == y2) {							// horizontal line
		// x1|y1 must be the left point
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = x2-x1;
		for(i=0; i<=length; i++) {
			glcd_Dot(x1+i, y1, mode);
		}
	} else {
		// x1 must be smaller than x2
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1)) {	// angle larger or equal 45
			length = x2-x1;								// not really the length :)
			m = ((y2-y1)*200)/length;
			yAlt = y1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+y1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				glcd_Line(x1+i, yAlt, x1+i, y, mode ); /* wuff wuff recurs. */
				if(length<=(y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length<=(y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		} else {										// angle smaller 45
			// y1 must be smaller than y2
			if(y1 > y2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}
			length = y2-y1;
			m = ((x2-x1)*200)/length;
			yAlt = x1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+x1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				glcd_Line(yAlt, y1+i, y, y1+i, mode); 
				if(length<=(x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length<=(x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		}
	}
}

/*********************************************************************
 * void glcd_Rect(unsigned char x, unsigned char y, 
 					unsigned char width, unsigned char height,
					unsigned char mode)  
 * PreCondition:    LCD initialized
 * Input:           x,y - top left corner
 *					width, height - width and height of the rectangle
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a rectangle
 * Note:            -
 ********************************************************************/
void glcd_Rect(unsigned char x, unsigned char y, unsigned char width, unsigned char height,
		unsigned char mode) 
{
	width--;
	height--;
	glcd_Line(x, y, x+width, y, mode);		// top
	glcd_Line(x, y, x, y+height, mode);	// left
	glcd_Line(x, y+height, x+width, y+height, mode);	// bottom
	glcd_Line(x+width, y, x+width, y+height, mode);	// right
}

/*********************************************************************
 * void glcd_Box(unsigned char x, unsigned char y, 
 					unsigned char width, unsigned char height,
					unsigned char mode)  
 * PreCondition:    LCD initialized
 * Input:           x,y - top left corner
 *					width, height - width and height of the box
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a solid box
 * Note:            -
 ********************************************************************/
void glcd_Box(unsigned char x, unsigned char y, unsigned char width, unsigned char height,
		unsigned char mode) 
{
	unsigned char i;
	
	if (!width) return; 
	
	width--;
	
	for (i=y;i<y+height;i++) 
		glcd_Line(x, i, x+width, i, mode);
}

/*********************************************************************
 * void glcd_Circle(unsigned char xCenter, unsigned char yCenter, 
					unsigned char radius, unsigned char mode)   
 * PreCondition:    LCD initialized
 * Input:           xCenter,yCenter - circle center
 *					radius - circle radius
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a circle 
 * Note:            -
 ********************************************************************/
void glcd_Circle(unsigned char xCenter, unsigned char yCenter, 
		unsigned char radius, unsigned char mode) 
{
  	int tSwitch, y, x = 0;
  	unsigned char d;

  	d = yCenter - xCenter;
  	y = radius;
  	tSwitch = 3 - 2 * radius;
  	
  	while (x <= y) {
    	glcd_Dot(xCenter + x, yCenter + y, mode);
    	glcd_Dot(xCenter + x, yCenter - y, mode);
    	
    	glcd_Dot(xCenter - x, yCenter + y, mode);
    	glcd_Dot(xCenter - x, yCenter - y, mode);
    	
    	glcd_Dot(yCenter + y - d, yCenter + x, mode);
    	glcd_Dot(yCenter + y - d, yCenter - x, mode);
    	
    	glcd_Dot(yCenter - y - d, yCenter + x, mode);
    	glcd_Dot(yCenter - y - d, yCenter - x, mode);

    	if (tSwitch < 0) {
    		tSwitch += (4 * x + 6);
    	} else {
      		tSwitch += (4 * (x - y) + 10);
      		y--;
    	}
    	x++;
  	}
}


/*********************************************************************
 * void glcd_Glyph(unsigned char left, unsigned char top,
			   		unsigned char width, unsigned char height,
			   		const unsigned char *glyph, unsigned char store_width)  
 * PreCondition:    LCD initialized
 * Input:           column    - x corrdinate of the left part of glyph          
    				row       - y coordinate of the top part of glyph       
    				width  	  - size in pixels of the width of the glyph    
    				height 	  - size in pixels of the height of the glyph   
    				glyph     - an unsigned char pointer to the glyph pixels 
                				to write assumed to be of length "width"
 * Output:          -
 * Overview:        Writes a glyph("letter") to the display at location x,y 
 * Note:            -
 ********************************************************************/

void glcd_Glyph(unsigned char left, unsigned char top,
			   unsigned char width, unsigned char height,
			   const unsigned char *glyph, unsigned char store_width)
{
	unsigned char bit_pos;
	unsigned char byte_offset;
	unsigned char y_bits;
	unsigned char remaining_bits;
	unsigned char mask;
	unsigned char char_mask;
	unsigned char x;
	const unsigned char *glyph_scan;
	unsigned char glyph_offset;

  	bit_pos = top & 0x07;		/* get the bit offset into a byte */

	glyph_offset = 0;			/* start at left side of the glyph rasters */
    char_mask = 0x80;			/* initial character glyph mask */

  	for (x = left; x < (left + width); x++)
  	{
    	byte_offset = top >> 3;        	/* get the byte offset into y direction */
		y_bits = height;				/* get length in y direction to write */
		remaining_bits = 8 - bit_pos;	/* number of bits left in byte */
		mask = glcd_MaskArray[bit_pos];	/* get mask for this bit */
		glyph_scan = glyph + glyph_offset;	 /* point to base of the glyph */

    	/* boundary checking here to account for the possibility of  */
    	/* write past the bottom of the screen.                        */
    	while((y_bits) && (byte_offset < GLCD_Y_BYTES)) /* while there are bits still to write */
    	{
			/* check if the character pixel is set or not */
			if(*glyph_scan & char_mask)
			{
				glcd_Buffer[byte_offset][x] |= mask;	/* set image pixel */
			}
			else
			{
      			glcd_Buffer[byte_offset][x] &= ~mask;	/* clear the image pixel */
			}

			if(glcd_MaskArray[0] & 0x80)
			{
				mask >>= 1;
			}
			else
			{
				mask <<= 1;
			}
			
			y_bits--;
			remaining_bits--;
      		if(remaining_bits == 0)
      		{
				/* just crossed over a byte boundry, reset byte counts */
				remaining_bits = 8;
				byte_offset++;
				mask = glcd_MaskArray[0];
      		}

			/* bump the glyph scan to next raster */
			glyph_scan += store_width;
		}

		/* shift over to next glyph bit */
		char_mask >>= 1;
		if(char_mask == 0)				/* reset for next byte in raster */
		{
			char_mask = 0x80;
			glyph_offset++;
	    }
	}
}

/*********************************************************************
 * glcd_Text(unsigned char left, unsigned char top, 
   unsigned char font, const char *str)  
 * PreCondition:    LCD initialized
 * Input:           left       coordinate of left start of string.                
					top        coordinate of top of string.
					font       font number to use for display (see fonts.h)
					str   	   text string to display (null-terminted)
 * Output:          -
 * Overview:        Prints the given string at location x,y in the specified font.
					Prints each character given via calls to lcd_glyph. 
 * Note:            The entry string is null terminated and 
 					non 0x20->0x7e characters are ignored.
 ********************************************************************/
void glcd_Text(unsigned char left, unsigned char top, unsigned char font, const char *str)
{
  	unsigned char x = left;
  	unsigned char glyph;
  	unsigned char width;
	unsigned char height;
	unsigned char store_width;
	const unsigned char *glyph_ptr;

  	while(*str != 0x00)
  	{
    	glyph = (unsigned char)*str;

		/* check to make sure the symbol is a legal one */
		/* if not then just replace it with the default character */
		if((glyph < fonts[font].glyph_beg) || (glyph > fonts[font].glyph_end))
		{
			glyph = fonts[font].glyph_def;
		}

    	/* make zero based index into the font data arrays */
    	glyph -= fonts[font].glyph_beg;
    	width = fonts[font].fixed_width;	/* check if it is a fixed width */
		if(width == 0)
		{
			width=fonts[font].width_table[glyph];	/* get the variable width instead */
		}

		height = fonts[font].glyph_height;
		store_width = fonts[font].store_width;

		glyph_ptr = fonts[font].glyph_table + ((unsigned int)glyph * (unsigned int)store_width * (unsigned int)height);

		/* range check / limit things here */
		if(x > GLCD_RIGHT)
		{
			x = GLCD_RIGHT;
		}
		if((x + width) > GLCD_RIGHT+1)
		{
			width = GLCD_RIGHT - x + 1;
		}
		if(top > GLCD_BOTTOM)
		{
			top = GLCD_BOTTOM;
		}
		if((top + height) > GLCD_BOTTOM+1)
		{
			height = GLCD_BOTTOM - top + 1;
		}

		glcd_Glyph(x,top,width,height,glyph_ptr,store_width);  /* plug symbol into buffer */

		x += width;							/* move right for next character */
		str++;								/* point to next character in string */
	}
}



void glcd_backlight(int state)
{
  if (state) 
  	hal_LCD_backlight(1);
  else
    hal_LCD_backlight(0);
}

