/*******************************************************************************/
/**	Description: 															  **/
/**		glcd.h - Graphical LCD driver  	header file							  **/
/*******************************************************************************/
/**  Compiler	: 	Keil CARM V2.4											  **/
/**	 Target 	:	Philips LPC2138											  **/
/**																			  **/
/**	 Code portions Copyright (C) 2004 Martin THOMAS							  **/
/**  Author		: 	Marko Pavlin, 											  **/
/**					marko.pavlin@guest.arnes.si								  **/
/*******************************************************************************/
/**	Revision history: 														  **/
/**	-----------------														  **/
/**	  v0.0 21.12.2005 - file created										  **/
/**	  v1.1 3.2.2006 - Sode cleanup and comments added						  **/
/**																			  **/
/**																			  **/
/*******************************************************************************/
 
#ifndef __GLCD_H__
#define __GLCD_H__

#ifndef BIT														 
	#define BIT(n)              (1 << (n))			 
#endif

/* Global defines for LCD controller */
#define GLCD_MAXPAGE 		 8
#define GLCD_MAXADDRESS		 128

#define GLCD_LEFT			0
#define GLCD_TOP			0
#define GLCD_RIGHT			(GLCD_MAXADDRESS-1)
#define GLCD_BOTTOM			(GLCD_MAXPAGE*8-1)

#define GLCD_MODE_NORMAL	0
#define GLCD_MODE_INVERTED  1

#define GLCD_MODE_CLEAR     0
#define GLCD_MODE_SET       1
#define GLCD_MODE_XOR       2

#define GLCD_CLEAR_BUFFER   0
#define GLCD_CLEAR_ALL      1

/*********************************************************************
 * Function:        glcd_Init(void)
 * PreCondition:    SPI and RTOS initialized
 * Input:           - 
 * Output:          -
 * Overview:        Initialize LCD module
 * Note:            Reset pause is implemented with os_dly_wait()
 ********************************************************************/
void glcd_Init(void);


/*********************************************************************
 * Function:        glcd_SetInvert(unsigned char mode)
 * PreCondition:    LCD initialized
 * Input:           mode (GLCD_MODE_INVERT or GLCD_MODE_NORMAL) 
 * Output:          -
 * Overview:        Set writing mode 
 * Note:            -
 ********************************************************************/
void glcd_SetInvert(unsigned char mode);


/*********************************************************************
 * Function:        glcd_Update(unsigned int top, unsigned int bottom)
 * PreCondition:    LCD initialized
 * Input:           top     top line of area to update.         
 *					bottom  bottom line of area to update.
 * Output:          -
 * Overview:        Copy buffer-RAM to display-RAM 
 * Note:            -
 ********************************************************************/
void glcd_Update(unsigned int top, unsigned int bottom);
#define glcd_UpdateAll() glcd_Update(GLCD_TOP, GLCD_BOTTOM);


/*********************************************************************
 * Function:        glcd_ClrScr(unsigned char bufferAndDisplay)
 * PreCondition:    LCD initialized
 * Input:           bufferAndDisplay (boolean), if 1 = clear LCD too  
 * Output:          -
 * Overview:        clear LCD buffer and display 
 * Note:            -
 ********************************************************************/
void glcd_ClrScr(unsigned char bufferAndDisplay);


/*********************************************************************
 * Function:        glcd_Dot(unsigned char x, unsigned char y, unsigned char mode)
 * PreCondition:    LCD initialized
 * Input:           x,y - dot coordinate         
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Set a single dot within LCD buffer
 * Note:            -
 ********************************************************************/
void glcd_Dot(unsigned char x, unsigned char y, unsigned char mode);
#define glcd_SetDot(x,y) glcd_Dot(x, y, GLCD_MODE_SET)
#define glcd_ClearDot(x,y) glcd_Dot(x, y, GLCD_MODE_CLEAR);


/*********************************************************************
 * Function:        void glcd_Line( unsigned char x1, unsigned char y1, 
						unsigned char x2, unsigned char y2,
						unsigned char mode )  
 * PreCondition:    LCD initialized
 * Input:           x1,y1 - line start coordinate
 *					x2,y2 - line end coordinate
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a line 
 * Note:            -
 ********************************************************************/
void glcd_Line(unsigned char x1, unsigned char y1, 
	unsigned char x2, unsigned char y2, unsigned char mode);


/*********************************************************************
 * void glcd_Circle(unsigned char xCenter, unsigned char yCenter, 
					unsigned char radius, unsigned char mode)   
 * PreCondition:    LCD initialized
 * Input:           xCenter,yCenter - circle center
 *					radius - circle radius
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a circle 
 * Note:            -
 ********************************************************************/
void glcd_Circle(unsigned char xCenter, unsigned char yCenter, 
		unsigned char radius, unsigned char mode);


/*********************************************************************
 * void glcd_Rect(unsigned char x, unsigned char y, 
 					unsigned char width, unsigned char height,
					unsigned char mode)  
 * PreCondition:    LCD initialized
 * Input:           x,y - top left corner
 *					width, height - width and height of the rectangle
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a rectangle
 * Note:            -
 ********************************************************************/
void glcd_Rect(unsigned char x, unsigned char y, unsigned char width, unsigned char height,
		unsigned char mode); 


/*********************************************************************
 * void glcd_Box(unsigned char x, unsigned char y, 
 					unsigned char width, unsigned char height,
					unsigned char mode)  
 * PreCondition:    LCD initialized
 * Input:           x,y - top left corner
 *					width, height - width and height of the box
 *					mode - GLCD_MODE_SET, GLCD_MODE_CLEAR, GLCD_MODE_XOR
 * Output:          -
 * Overview:        Draw a solid box
 * Note:            -
 ********************************************************************/
void glcd_Box(unsigned char x, unsigned char y, unsigned char width, unsigned char height,
		unsigned char mode); 


/*********************************************************************
 * glcd_Text(unsigned char left, unsigned char top, 
   unsigned char font, const char *str)  
 * PreCondition:    LCD initialized
 * Input:           left       coordinate of left start of string.                
					top        coordinate of top of string.
					font       font number to use for display (see fonts.h)
					str   	   text string to display (null-terminted)
 * Output:          -
 * Overview:        Prints the given string at location x,y in the specified font.
					Prints each character given via calls to lcd_glyph. 
 * Note:            The entry string is null terminated and 
 					non 0x20->0x7e characters are ignored.
 ********************************************************************/
void glcd_Text(unsigned char left, unsigned char top, unsigned char font, const char *str);


/*********************************************************************
 * Function:        glcd_WriteCommand(unsigned char byte)
 * PreCondition:    LCD initialized
 * Input:           command byte 
 * Output:          -
 * Overview:        Send command to LCD module
 * Note:            None
 ********************************************************************/
void glcd_WriteCommand(unsigned char byte);


/*********************************************************************
 * Function:        glcd_backlight(BOOL state)
 * PreCondition:    -
 * Input:           LED status 
 * Output:          -
 * Overview:        Switch backlight ON/OFF
 * Note:            None
 ********************************************************************/
void glcd_backlight(int state);




#endif


